# vim:fileencoding=utf-8
# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.testing
import gocept.directory.app
import os.path
import unittest
import xmlrpclib
import zope.app.testing.functional
import zope.app.testing.xmlrpc


class TestRing1(zope.app.testing.functional.FunctionalTestCase):

    layer = gocept.directory.testing.DataLayer

    def setUp(self):
        super(TestRing1, self).setUp()
        self.directory = directory = self.getRootFolder()['directory']
        self.old = zope.app.component.hooks.getSite()
        zope.app.component.hooks.setSite(self.directory)

        newyork = gocept.directory.location.Location()
        newyork.abbreviation = 'ny'
        directory['newyork'] = newyork

        stampy = gocept.directory.nodes.PhysicalMachine()
        stampy.name = 'stampy'
        stampy.directory_ring = 1
        stampy.directory_password = 'gfhdjk'
        stampy.location = newyork

        directory['rg-services']['stampy'] = stampy

        self.rpc = gocept.directory.testing.ServerProxy(
            'http://localhost/directory/',
            user='stampy', password='gfhdjk')

    def tearDown(self):
        zope.app.component.hooks.setSite(self.old)

    def test_lookup_node_ring1_allowed(self):
        getattr(self.rpc, 'rg-services').lookup_node('stampy')

    def test_lookup_node_ring1_denied(self):
        # A node that doesn't exist
        self.assertRaises(xmlrpclib.ProtocolError,
                          getattr(self.rpc, 'rg-services').lookup_node, 'foo')
        # A node from a different resource group
        self.directory['rg-foo'] = (
            gocept.directory.resourcegroup.ResourceGroup())
        p = gocept.directory.nodes.PhysicalMachine()
        p.puppet_classes = []
        p.name = 'bar'
        p.contacts = 'asdf'
        p.environment = 'development'
        p.directory_password = ''
        p.directory_ring = ''
        p.location = self.directory['newyork']
        self.directory['rg-foo']['bar'] = p
        self.assertRaises(
            xmlrpclib.ProtocolError,
            getattr(self.rpc, 'rg-foo').lookup_node, 'bar')


def test_suite():
    return unittest.makeSuite(TestRing1)
