# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import persistent
import zope.location.location
import zope.interface
import zope.app.authentication.interfaces
import gocept.directory.interfaces
import grok


MAX_RING = 1


@grok.subscribe(
    zope.app.authentication.interfaces.IAuthenticatedPrincipalCreated)
def setup_host_groups(event):
    # Activate Ring0/Ring1 group for machine logins
    if not isinstance(event.info, MachinePrincipalInfo):
        return
    node = zope.component.getUtility(
        gocept.directory.interfaces.INode, name=event.info.id)
    ring = node.directory_ring
    for i in range(ring, MAX_RING + 1):
        event.principal.groups.append('gocept.directory.Ring%s' % i)


class MachinePrincipalInfo(object):

    zope.interface.implements(
        zope.app.authentication.interfaces.IPrincipalInfo)

    def __init__(self, name):
        self.id = name
        self.title = name
        self.description = name
        self.credentialsPlugin = None
        self.authenticatorPlugin = None


class Account(object):

    def __init__(self, login, password):
        self.login = login
        self.password = password


class MachineLogin(persistent.Persistent,
                   zope.location.location.Location):
    """Handles machine logins for Ring0 and Ring1 access."""

    zope.interface.implements(
        zope.app.authentication.interfaces.IAuthenticatorPlugin)

    def authenticateCredentials(self, credentials):
        if not isinstance(credentials, dict):
            return None
        if not ('login' in credentials and 'password' in credentials):
            return None
        login = credentials['login']

        account = self.getAccount(login)
        if account is None:
            return
        if account.password != credentials['password']:
            return

        return MachinePrincipalInfo(login)

    def principalInfo(self, id):
        return self.getAccount(id)

    def getAccount(self, login):
        if '@' in login:
            return
        try:
            node = zope.component.getUtility(
                gocept.directory.interfaces.INode,
                name=login)
        except LookupError:
            return
        return Account(login, node.directory_password)
